<?xml version="1.0"?> 
<xsl:stylesheet version="1.0"
        xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!-- ===============================================================
        mpp2planner.xsl

        Conversion between Microsoft Project Plan (MPP) and Planner XML files

        Copyright (c) 2004 Kurt Maute (kurt@maute.us)
        Copyright (c) 2007 Nguyễn Thái Ngọc Duy (pclouds@gmail.com)
     ===============================================================-->

<xsl:output method="xml" indent="yes"/>

<!-- ===============================================================
        Main  -->

<xsl:template match="mpp[@version='9']">

<!--  Project Tag  -->
  <xsl:variable name="projstartdate">
    <xsl:call-template name="ms2pdate">
      <xsl:with-param name="thedate" select="document-summary/item[@name='Start']/@value"/>
    </xsl:call-template>
  </xsl:variable>
  <project name="{summary/item[@name='dc:title']/@value}" manager=""  project-start="{$projstartdate}" mrproject-version="2" calendar="1">


<!--  Properties  -->
    <properties>
      <property name="cost" type="cost" owner="resource" label="Cost" 
      description="standard cost for a resource"/>
    </properties>

<!--  Phases  - not implemented  -->

<!--  Calendars  - only writing standard Planner calendar right now,
        since the MS Project implementation is painfully dissimilar  -->
  <calendars>
    <day-types>
      <day-type id="0" name="Working" description="A default working day"/>
      <day-type id="1" name="Nonworking" description="A default non working day"/>
      <day-type id="2" name="Use base" description="Use day from base calendar"/>
    </day-types>
    <calendar id="1" name="Default">
      <default-week mon="0" tue="0" wed="0" thu="0" fri="0" sat="1" sun="1"/>
      <overridden-day-types>
        <overridden-day-type id="0">
          <interval start="0800" end="1200"/>
          <interval start="1300" end="1700"/>
        </overridden-day-type>
      </overridden-day-types>
      <days/>
    </calendar>
  </calendars>

<!--  Tasks    -->
    <xsl:apply-templates select="project/tasks"/>

<!--  Resources    -->
    <xsl:apply-templates select="project/resources"/>

<!--  Allocations    -->
    <xsl:apply-templates select="project/assignments"/>

  </project>
</xsl:template>

<!-- ===============================================================
        Task Handling Templates  -->

<!-- Main Task loop 
     This template loops thru the 1st outline level of task elements
     and calls the recursive template 'task' to handle the subsequent
     levels.  Couldn't just use the recursive routine for all since
     starts-with() can't match on an empty variable - so we need this
     to get going.  -->
<xsl:template match="tasks">
  <tasks>
  <xsl:apply-templates select="task"/>
  </tasks>
</xsl:template>

<xsl:template match="task">
  <task>
  <xsl:call-template name="write-task"/>
  </task>
</xsl:template>

<!-- Write Task
     This template writes out all the task info, except for the task start
     and end tags.  -->
<xsl:template name="write-task">
  <xsl:param name="task-id" select="UniqueId/@value"/>
  <xsl:attribute name="id"><xsl:value-of select="UniqueId/@value"/></xsl:attribute>
  <xsl:attribute name="name"><xsl:value-of select="TASK_NAME/@value"/></xsl:attribute>
  <xsl:attribute name="note"><xsl:value-of select="TASK_NOTES/@value"/></xsl:attribute>
  <xsl:attribute name="work">
    <xsl:call-template name="ms2pduration">
      <xsl:with-param name="dur" select="Work"/>
    </xsl:call-template>
  </xsl:attribute>
  <xsl:attribute name="duration">
    <xsl:call-template name="ms2pduration">
      <xsl:with-param name="dur" select="Duration"/>
    </xsl:call-template>
  </xsl:attribute>
  <xsl:attribute name="start"><xsl:value-of select="Start/@value"/></xsl:attribute>
  <xsl:attribute name="end"><xsl:value-of select="Finish/@value"/></xsl:attribute>
  <xsl:attribute name="percent-complete">
    <xsl:value-of select="PercentageComplete/@value"/>
  </xsl:attribute>
  <xsl:attribute name="priority">
    <xsl:value-of select="Priority/@value"/>
  </xsl:attribute>
  <xsl:attribute name="type">
    <xsl:choose>
      <xsl:when test="milestone/@value='false'">normal</xsl:when>
      <xsl:otherwise>milestone</xsl:otherwise>
    </xsl:choose>
  </xsl:attribute>
  <xsl:attribute name="scheduling">
    <xsl:choose>
      <xsl:when test="Type/@value=0">fixed-duration</xsl:when> <!-- should be fixed-units but we don't have that -->
      <xsl:when test="Type/@value=1">fixed-duration</xsl:when>
      <xsl:otherwise>fixed-work</xsl:otherwise>
    </xsl:choose>
  </xsl:attribute>

  <!-- Constraints handled here -->
  <xsl:choose>
    <xsl:when test="ConstraintType/@value='MSO'">
      <xsl:attribute name="type">must-start-on</xsl:attribute>
      <xsl:attribute name="time"><xsl:value-of select="ConstraintDate/@value"/></xsl:attribute>
    </xsl:when>
    <xsl:when test="ConstraintType/@value='SNET'">
      <xsl:attribute name="type">start-no-earlier-than</xsl:attribute>
      <xsl:attribute name="time"><xsl:value-of select="ConstraintDate/@value"/></xsl:attribute>
    </xsl:when>
    <xsl:when test="ConstraintType/@value='FNLT'">
      <xsl:attribute name="type">finish-no-later-than</xsl:attribute>
      <xsl:attribute name="time"><xsl:value-of select="ConstraintDate/@value"/></xsl:attribute>
    </xsl:when>
  </xsl:choose>

  <!-- Predecessors handled here -->
  <predecessors>
  <xsl:for-each select="/mpp/project/constraints/constraint[@task2=$task-id]">
    <predecessor id="{@constraint-id}" predecessor-id="{@task1}">
      <xsl:attribute name="type">
        <xsl:choose>
	  <xsl:when test="@type='unknown'">FS</xsl:when>
	  <xsl:otherwise><xsl:value-of select="@type"/></xsl:otherwise>
	</xsl:choose>
      </xsl:attribute>
      <!--
      <xsl:attribute name="lag">
        <xsl:value-of select="ms:LinkLag*6"/>
      </xsl:attribute>
      -->
    </predecessor>
  </xsl:for-each>
  </predecessors>

  <xsl:apply-templates select="task"/>
</xsl:template>

<!-- ===============================================================
        Resource Handling Template  -->

<!-- Resource loop   -->
<xsl:template match="resources">
  <resources>
    <xsl:apply-templates select="resource[UniqueId/@value>0]"/>
  </resources>
</xsl:template>

<xsl:template match="resource">
  <xsl:param name="id" select="UniqueId/@value"/>
  <resource id="{$id}" name="{//resources/RESOURCE_NAME[@unique-id=$id]/@value}" short-name="{//resources/RESOURCE_INITIALS[@unique-id=$id]/@value}">
    <xsl:attribute name="type">
      <xsl:choose>
        <xsl:when test="Type/@value='material'">2</xsl:when>
        <xsl:otherwise>1</xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
    <xsl:attribute name="units">
      <xsl:value-of select='MaxUnits/@value'/>
    </xsl:attribute>
    <xsl:attribute name="email">
      <xsl:value-of select='//resources/RESOURCE_EMAIL[@unique-id=$id]/@value'/>
    </xsl:attribute>
    <xsl:attribute name="note">
      <xsl:value-of select='//resources/RESOURCE_NOTES[@unique-id=$id]/@value'/>
    </xsl:attribute>
    <xsl:attribute name="std-rate">
      <xsl:value-of select='StandardRate/@value'/>
    </xsl:attribute>
    <xsl:attribute name="ovt-rate">
      <xsl:value-of select='OvertimeRate/@value'/>
    </xsl:attribute>
    <properties>
      <property name="cost" value="{Cost/@value}"/>
    </properties>
  </resource>
</xsl:template>

<!-- ===============================================================
        Allocations Handling Template  -->

<!-- Allocations loop   -->
<xsl:template match="assignments">
  <allocations>
    <xsl:for-each select="assignment[ResourceId/@value >= 0]">
      <allocation>
      <xsl:attribute name="task-id">
        <xsl:value-of select="TaskId/@value"/>
      </xsl:attribute>
      <xsl:attribute name="resource-id">
        <xsl:value-of select="ResourceId/@value"/>
      </xsl:attribute>
      <xsl:attribute name="units">
        <xsl:value-of select="Units/@value"/>
      </xsl:attribute>
      </allocation>
    </xsl:for-each>
  </allocations>
</xsl:template>

<!-- ===============================================================
        Global Functions  -->

<!-- Convert date from MSP to Planner format -->
<xsl:template name="ms2pdate">
  <xsl:param name="thedate"/>
  <xsl:variable name="formatted" select="concat(translate(substring($thedate, 
   1, 11),'-',''),translate(substring-after($thedate,'T'),':',''))"/>
  <xsl:value-of select="$formatted"/>
</xsl:template>

<!-- Convert duration from MSP to Planner format -->
<xsl:template name="ms2pduration">
  <xsl:param name="dur"/>
    <xsl:choose>
      <xsl:when test="$dur/@unit='hours'"><xsl:value-of select="$dur/@value div 600 * 60 * 60"/></xsl:when>
      <xsl:when test="$dur/@unit='elapsed-hours'"><xsl:value-of select="$dur/@value div 600 * 60 * 60"/></xsl:when>
      <xsl:when test="$dur/@unit='minutes'"><xsl:value-of select="$dur/@value div 10 * 60"/></xsl:when>
      <xsl:when test="$dur/@unit='elapsed-minutes'"><xsl:value-of select="$dur/@value div 10 * 60"/></xsl:when>
      <xsl:otherwise><xsl:value-of select="$dur/@value div 4800 * 8 * 60 * 60" /></xsl:otherwise>
    </xsl:choose>
</xsl:template>


<!-- ===============================================================
        End  -->

</xsl:stylesheet>
